from __future__ import annotations

from typing import List

from .SteadyStateSubspaceFrequency import SteadyStateSubspaceFrequency

SteadyStateSubspaceFrequencyArray = List[SteadyStateSubspaceFrequency]
